C   Linearized Hemispheric Primitive Model
C
C     Initial value/response problem version
C
C     by Naoki Sato  February 23, 2004
C     revised by Naoki Sato  December 10, 2004
C     revised by Naoki Sato  January 30, 2006

C ########################################
C      SETTING THE PARAMETERS
C ########################################

      PARAMETER (LX=144,MX=36,NX=11)

      PARAMETER (IX=480*20,IO=480,DTM=0.180E3)

      REAL DX(MX),DY(MX),FC(MX),DZ(NX),PLEV(NX),TPG(LX,MX),
     +     U0(LX,MX,NX),V0(LX,MX,NX),T0(LX,MX,NX),P0(LX,MX),
     +     U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX),
     +     H0(LX,MX,NX),H (LX,MX,NX),
     +     W0(LX,MX,NX),W (LX,MX,NX),Z (LX,MX,NX),
     +     EX(LX,MX,NX),RTG(LX,MX,NX),
     +     DM1(LX,MX,NX),DM2(LX,MX+1,NX),DM3(LX,MX+1),DM4(LX,MX)

      CHARACTER CM*1

C ########################################
C   1. PREPARATION
C ########################################

C #### Calculation mode ####
C   CM = 'L': Linear model
C   CM = 'M': Non-linear model around the basic field
C             (The basic field is exactly maintained if no external
C              forcing.)
C   CM = 'N': Non-linear model (under construction)
      CM = 'L'
C      CM = 'M'
C      CM = 'N'

      CALL PARAM (LX,MX,NX,DX,DY,DZ,PLEV0,PLEV,FC)
      CALL BASIC (LX,MX,NX,DZ,PLEV0,PTOP,U0,V0,T0,P0,TPG)

      CALL DIAGX (LX,MX,NX,DZ,PTOP,P0,EX)
      CALL DIAGH (LX,MX,NX,DZ,'I',TPG,PTOP,P0,EX,T0,H0)
      CALL DIAGR (LX,MX,NX,DZ,PTOP,P0,T0,RTG)
      CALL DIAGW (LX,MX,NX,DX,DY,DZ,'I',PTOP,P0,U0,V0,P,U,V,W0,DM4)
      CALL BSCPRN (LX,MX,NX,DZ,PLEV,H0,U0,T0)
      CALL INITZ (LX,MX,NX,U,V,T,P,UP,VP,TP,PP)

C #### Subroutine for initial value problem ####
C   For initial value problem, remove the comment of the following line.
C      CALL FORCE1 (LX,MX,NX,DZ,20.,130.,10.,0.15,0.75,2.,T,TP)

C   Output files are opened.

      CALL OFLOPN (LX,MX,NX)

C #### Do loop 11 starts here ####

C   In roop 11, the time derivateves of U, V, T and P are calculated,
C   and time-integrations are performed for them.

      WRITE(6,*) 'Time-integration:'
      WRITE(6,'(1X,A22,1X,F9.3,1X,A4)')
     +                          'Time step            =',DTM,'sec.'
      WRITE(6,'(1X,A22,1X,I6)') 'Number of time steps =',IX
      WRITE(6,'(1X,A22,1X,I6)') 'Interval of output   =',IO
      WRITE(6,'(1X,A22,1X,I6)') 'Number of outputs    =',IX/IO
      WRITE(6,*) '===================================='
      WRITE(6,*) 'Time-integration started...'

      DO 11 I=1,IX

C ########################################
C   2. CALCULATION OF DEPENDENT VARIABLES
C ########################################

      CALL DIAGW (LX,MX,NX,DX,DY,DZ,CM,PTOP,P0,U0,V0,P,U,V,W,DM4)
      CALL DIAGH (LX,MX,NX,DZ,CM,TPG,PTOP,P0,EX,T,H)

C ########################################
C   3. DYNAMICAL PROCESSES
C ########################################

      CALL TENDZ (LX,MX,NX,DU,DV,DT,DP)
      CALL CNVRG (LX,MX,NX,DX,DY,DZ,CM,PTOP,P0,U0,V0,P,U,V,DP)
      CALL ADVEC (LX,MX,NX,DX,DY,DZ,CM,
     +            U0,V0,W0,T0,U,V,W,T,DU,DV,DT)
      CALL PGRAD (LX,MX,NX,DX,DY,DZ,CM,PTOP,P0,H0,P,H,RTG,DU,DV)
      CALL CORIO (LX,MX,NX,'H',CM,FC,U0,V0,U,V,DU,DV)
      CALL INTEL (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      CALL POLES (LX,MX,NX,U,V,T,P)

C ########################################
C   4. PHYSICAL PROCESSES
C ########################################

      CALL TENDZ (LX,MX,NX,DU,DV,DT,DP)
      CALL DIFFU (LX,MX,NX,DX,DY,DZ,RTG,CM,U0,V0,T0,U,V,T,DU,DV,DT)
      CALL DMPNG (LX,MX,NX,U,V,T,DU,DV,DT)
      CALL SPONG (LX,MX,NX,U,V,T,DU,DV,DT)

C #### Subroutine for response problem ####
C   For response problem, remove the comment of the following line.
      CALL FORCE2 (LX,MX,NX,DZ,20.,130.,10.,0.15,0.75,2.*1.0E-5,DT)

      CALL INTEE (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      CALL POLES (LX,MX,NX,U,V,T,P)

C ########################################
C   5. OUTPUT
C ########################################

      IF (MOD(I,IO).EQ.0) THEN
        CALL DIAGH (LX,MX,NX,DZ,CM,TPG,PTOP,P0,EX,T,H)
        CALL DIAGZ (LX,MX,NX,RTG,P0,P,H,Z)
        WRITE(6,'(1X,I4,1X,A1,1X,I4)') I/IO,'/',IX/IO
        CALL WRDAT (LX,MX,NX,I/IO,U,V,T,Z,P,DM2,DM3)
      ENDIF

   11 CONTINUE

C #### Do loop 11 ends here ####

      CLOSE(11)
      CLOSE(12)
      CLOSE(13)
      CLOSE(14)
      CLOSE(15)

      STOP
      END

C ########################################
C   SUBROUTINES
C ########################################

C ========================================
C   BASIC FIELDS
C ========================================

      SUBROUTINE PARAM (LX,MX,NX,DX,DY,DZ,PLEV0,PLEV,FC)
      REAL DX(MX),DY(MX),DZ(NX),PLEV(NX),FC(MX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
        DX(M) = 0.5E0 * PI*RE/REAL(MX)
     +          * COS(0.5E0*PI*(REAL(M)-0.5E0)/REAL(MX))
        DY(M) = 0.5E0 * PI*RE/REAL(MX)
        FC(M) = 0.2E1 * OM * SIN(0.5E0*PI*(REAL(M)-0.5E0)/REAL(MX))
   11 CONTINUE
      Z1 = 0.100E0
      Z2 = 1.000E0
      DO 21 N=1,NX
        R = (Z2/Z1)**(1.0E0/REAL(NX))
        DZ(N) = Z1*(R-1.0E0)*R**(N-1)
   21 CONTINUE
      PS = 0.101325E4
      Z = Z1
      PLEV0 = PS * Z1
      DO 22 N=1,NX
        Z = Z + 0.5E0*DZ(N)
        PLEV(N) = PS * Z
        Z = Z + 0.5E0*DZ(N)
   22 CONTINUE
      RETURN
      END

      SUBROUTINE BASIC (LX,MX,NX,DZ,PLEV0,PTOP,U,V,T,P,H)
      REAL PSTAN(17),DZ(NX),
     +     PDAT(144,73),PLEVS(144,73,999),P(LX,MX),
     +     UDAT(144,73,17),UU(144,73,999),U(LX,MX,NX),
     +     VDAT(144,73,17),VV(144,73,999),V(LX,MX,NX),
     +     TDAT(144,73,17),TT(144,73,999),T(LX,MX,NX),
     +     HDAT(144,73),H(LX,MX)
      DATA PSTAN/1000.,925.,850.,700.,600.,500.,400.,300.,250.,200.,
     +           150.,100.,70.,50.,30.,20.,10/
      CHARACTER INFL*80
      WRITE(6,*) 'Basic field data files are automatically designated:'
  101 FORMAT (1X,A18,1X,A9)
C      INFL = 'TOPO.dat'
C      WRITE(6,101) 'Topography       =',INFL
C      CALL RDDAT (144,73, 1,INFL,HDAT)
      CALL ZERO  (144,73,1,HDAT)
      INFL = 'U0.dat'
      WRITE(6,101) 'Zonal wind       =',INFL
      CALL RDDAT (144,73,17,INFL,UDAT)
      INFL = 'V0.dat'
      WRITE(6,101) 'Meridional wind  =',INFL
      CALL RDDAT (144,73,17,INFL,VDAT)
      INFL = 'T0.dat'
      WRITE(6,101) 'Temperature      =',INFL
      CALL RDDAT (144,73,17,INFL,TDAT)
      INFL = 'P0.dat'
      WRITE(6,101) 'Surface pressure =',INFL
      CALL RDDAT (144,73, 1,INFL,PDAT)
      WRITE(6,*) '===================================='
      CALL THETA (144,73,17,PSTAN,TDAT)
      CALL PSMOD (144,73,17,PSTAN,PDAT,TDAT,HDAT)
      CALL PCOOR (144,73,NX,DZ,PLEV0,PDAT,PLEVS)
      CALL LAYER (144,73,17,NX,PSTAN,PLEVS,UDAT,UU)
      CALL LAYER (144,73,17,NX,PSTAN,PLEVS,VDAT,VV)
      CALL LAYER (144,73,17,NX,PSTAN,PLEVS,TDAT,TT)
      CALL RDCNVU (LX,MX,NX,UU,U)
      CALL RDCNVV (LX,MX,NX,VV,V)
      CALL RDCNVG (LX,MX,NX,TT,T)
      CALL RDCNVG (LX,MX,1,PDAT,P)
      CALL RDCNVG (LX,MX,1,HDAT,H)
      PTOP = 1.0E2*PLEV0
      DO 11 M=1,MX
      DO 12 L=1,LX
        P(L,M) = 1.0E2*P(L,M)
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      DO 22 L=1,LX
        V(L,1,N) = 0.0E0
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

      SUBROUTINE RDDAT (LX,MX,NNX,INFL,G)
      REAL G(LX,MX,NNX)
      CHARACTER INFL*80
      OPEN(10,FILE=INFL,STATUS='OLD',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*MX*NNX)
      READ(10,REC=1) G
      CLOSE(10)
      RETURN
      END

      SUBROUTINE LAYER (LX,MX,NNX,NX,PSTAN,PLEVS,GDAT,G)
      REAL PSTAN(NNX),PLEVS(LX,MX,999),GDAT(LX,MX,NNX),G(LX,MX,999)
      DO 11 M=1,MX
      DO 12 L=1,LX
      DO 21 N=1,NX
      P = PLEVS(L,M,N)
      DO 31 NN=NNX-1,1,-1
      P1 = PSTAN (NN+1)
      P2 = PSTAN (NN)
      IF ((P.LE.P2).OR.(NN.EQ.1)) THEN
        NN1 = NN+1
        NN2 = NN
        C1 = (P2-P)/(P2-P1)
        C2 = (P-P1)/(P2-P1)
      GO TO 39
      ENDIF
   31 CONTINUE
   39 CONTINUE
        G(L,M,N) = C1*GDAT(L,M,NN1) + C2*GDAT(L,M,NN2)
   21 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE RDCNVG (LX,MX,NX,G,GG)
      REAL G(144,73,999),GG(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      M1 = 37-M
      M2 = 38-M
      DO 13 L=1,LX
      L1 = L
      L2 = L+1
      IF (L2.GT.144) L2 = L2-144
        GG(L,M,N) = 0.25E0*(G(L1,M1,N)+G(L2,M1,N)+G(L1,M2,N)+G(L2,M2,N))
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE RDCNVU (LX,MX,NX,G,GG)
      REAL G(144,73,999),GG(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      M1 = 37-M
      M2 = 38-M
      DO 13 L=1,LX
      L1 = L
      L2 = L
        GG(L,M,N) = 0.25E0*(G(L1,M1,N)+G(L2,M1,N)+G(L1,M2,N)+G(L2,M2,N))
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE RDCNVV (LX,MX,NX,G,GG)
      REAL G(144,73,999),GG(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      M1 = 38-M
      M2 = 38-M
      DO 13 L=1,LX
      L1 = L
      L2 = L+1
      IF (L2.GT.144) L2 = L2-144
        GG(L,M,N) = 0.25E0*(G(L1,M1,N)+G(L2,M1,N)+G(L1,M2,N)+G(L2,M2,N))
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE THETA (LX,MX,NX,PSTAN,T)
      REAL PSTAN(NX),T(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        T(L,M,N) = T(L,M,N) * (1.0E3/PSTAN(N))**(0.2E1/0.7E1)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE PSMOD (LX,MX,NX,PSTAN,P,T,H)
      REAL PSTAN(NX),P(LX,MX),T(LX,MX,NX),H(LX,MX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
      DO 12 L=1,LX
        PR = 1.0E2*P(L,M)
        DPR = 1.0E2*(P(L,M)-PSTAN(1))
        TM = T(L,M,1) * (PR/1.0E5)**(0.2E1/0.7E1)
        Z = RC*TM/GC * DPR / PR
      IF (Z.GE.H(L,M)) THEN
        P(L,M) = ((Z-H(L,M))*P(L,M)+H(L,M)*PSTAN(1))
     +           / Z
        GO TO 29
      ENDIF
      DO 21 N=1,NX-1
        PR = 1.0E2*0.5E0*(PSTAN(N)+PSTAN(N+1))
        DPR = 1.0E2*(PSTAN(N)-PSTAN(N+1))
        TM = T(L,M,N) * (PR/1.0E5)**(0.2E1/0.7E1)
        Z1 = Z
        Z = Z + RC*TM/GC * DPR / PR
      IF (Z.GE.H(L,M)) THEN
        P(L,M) = ((Z-H(L,M))*PSTAN(N)+(H(L,M)-Z1)*PSTAN(N+1))
     +           / (Z-Z1)
        GO TO 29
      ENDIF
        P(L,M) = PSTAN(1)
   21 CONTINUE
   29 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE PCOOR (LX,MX,NX,DZ,PTOP,P,PLEVS)
      REAL DZ(NX),P(LX,MX),PLEVS(LX,MX,999)
      Z0 = 0.0E0
      DO 20 N=1,NX
        Z0=Z0+DZ(N)
   20 CONTINUE
      DO 11 N=1,NX 
        Z=0.0E0
      DO 21 NN=1,N-1
        Z=Z+DZ(NN)/Z0
   21 CONTINUE
        Z=Z+0.5E0*DZ(N)/Z0
      DO 12 M=1,MX
      DO 13 L=1,LX
        PLEVS(L,M,N) = (1.0E0-Z)*PTOP + Z*P(L,M)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE EQUAT (LX,MX,NX,U,V)
      REAL U(LX,MX,NX),V(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      C = REAL(IABS(M-1-(MX-1)/2))/REAL((MX-1)/2)*REAL(9)
      C = AMIN1(C,1.0E0)
      IF (C.NE.1.0E0) THEN
        SUM = 0.0E0
      DO 21 L=1,LX
        SUM = SUM + U(L,M,N)
   21 CONTINUE
        U0 = SUM / REAL(LX)
      DO 22 L=1,LX
        V(L,M,N) = C * V(L,M,N)
        U(L,M,N) = C * U(L,M,N) + (1.0E0-C) * U0
   22 CONTINUE
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE BSCPRN (LX,MX,NX,DZ,PLEV,H,U,T)
      REAL DZ(NX),PLEV(NX),H(LX,MX,NX),U(LX,MX,NX),T(LX,MX,NX)
      WRITE(6,*) 'Basic field data:'
      WRITE(6,'(1X,A5,1X,A5,1X,A5,1X,A6,1X,A6,2(1X,A6))')
     +  'Lyr #','d sgm','Pres.','Z(45N)','U(45N)','th(Eq)','th(NP)'
      DO 11 N=1,NX
        H0 = 0.0E0
        U0 = 0.0E0
        T1 = 0.0E0
        T2 = 0.0E0
      DO 21 L=1,LX
        H0 = H0 + 0.5E0*(H(L,18,N)+H(L,19,N))
        U0 = U0 + 0.5E0*(U(L,18,N)+U(L,19,N))
        T1 = T1 + T(L, 1,N)
        T2 = T2 + T(L,MX,N)
   21 CONTINUE
        H0 = H0 / REAL(LX)
        U0 = U0 / REAL(LX)
        T1 = T1 / REAL(LX)
        T2 = T2 / REAL(LX)
      WRITE(6,'(1X,I5,1X,F5.3,1X,F5.0,1X,F6.0,1X,F6.1,2(1X,F6.1))')
     +  N,DZ(N),PLEV(N),H0,U0,T1,T2
   11 CONTINUE
      WRITE(6,*) '===================================='
      RETURN
      END

C ========================================
C   BOUNDARY CONDITIONS
C ========================================

      SUBROUTINE SPONG (LX,MX,NX,U,V,T,DU,DV,DT)
      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      TAU = 0.5E5
      TAUI = 1.0E0/TAU
      DO 11 N=1,NX
      DO 12 L=1,LX
      DO 21 M=1,4
        C1 = (REAL(4-M)+0.5E0)/REAL(4)
        C2 = (REAL(4-M)+1.0E0)/REAL(4)
        DU(L,M,N) = DU(L,M,N) - C1 * TAUI * U(L,M,N)
        DV(L,M,N) = DV(L,M,N) - C2 * TAUI * V(L,M,N)
        DT(L,M,N) = DT(L,M,N) - C1 * TAUI * T(L,M,N)
   21 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE POLES (LX,MX,NX,U,V,T,P)
      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),P(LX,MX)
      CALL POLESG (LX,MX,NX,U)
      CALL POLESV (LX,MX,NX,V)
      CALL POLESG (LX,MX,NX,T)
      CALL POLESG (LX,MX, 1,P)
      RETURN
      END

      SUBROUTINE POLESG (LX,MX,NX,G)
      REAL G(LX,MX,NX)
      M1 = 12
      M2 = 6
      M3 = 3
      M4 = 2
      M5 = 1
      DO 11 M=MX-M1+1,MX-M2
        CALL POLESA (LX,MX,NX,M,2,G)
   11 CONTINUE
      DO 12 M=MX-M2+1,MX-M3
        CALL POLESA (LX,MX,NX,M,4,G)
   12 CONTINUE
      DO 13 M=MX-M3+1,MX-M4
        CALL POLESA (LX,MX,NX,M,8,G)
   13 CONTINUE
      DO 14 M=MX-M4+1,MX-M5
        CALL POLESA (LX,MX,NX,M,16,G)
   14 CONTINUE
      DO 15 M=MX,MX
        CALL POLESA (LX,MX,NX,M,LX,G)
   15 CONTINUE
      RETURN
      END

      SUBROUTINE POLESV (LX,MX,NX,G)
      REAL G(LX,MX,NX)
      M1 = 12
      M2 = 6
      M3 = 3
      M4 = 2
      M5 = 1
      DO 11 M=MX-M1+1,MX-M2
        CALL POLESA (LX,MX,NX,M+1,2,G)
   11 CONTINUE
      DO 12 M=MX-M2+1,MX-M3
        CALL POLESA (LX,MX,NX,M+1,4,G)
   12 CONTINUE
      DO 13 M=MX-M3+1,MX-M4
        CALL POLESA (LX,MX,NX,M+1,8,G)
   13 CONTINUE
      DO 14 M=MX-M4+1,MX-M5
        CALL POLESA (LX,MX,NX,M+1,16,G)
   14 CONTINUE
      RETURN
      END

      SUBROUTINE POLESA (LX,MX,NX,M,LR,G)
      REAL G(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 K=1,(LX-1)/LR+1
      L1 = 1+LR*(K-1)
      L2 = MIN0(LR+LR*(K-1),LX)
        SUM = 0.0E0
      DO 21 L=L1,L2
        SUM = SUM + G(L,M,N)
   21 CONTINUE
      DO 22 L=L1,L2
        G(L,M,N) = SUM / REAL(L2-L1+1)
   22 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE POLE2 (LX,MX,NX,U,V,T,P)
      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),P(LX,MX)
      REAL SINTH(144,144/2),COSTH(144,144/2)
      DATA NCHECK /0/
      SAVE NCHECK,SINTH,COSTH
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      IF (COSTH(1,1).EQ.0.0E0) THEN
      NCHECK=1
      DO 101 K=1,LX/2
      DO 102 L=1,LX
        TH = 0.2E1*PI*REAL(K)*REAL(L-1)/REAL(LX)
        SINTH(L,K) = SIN(TH)
        COSTH(L,K) = COS(TH)
  102 CONTINUE
  101 CONTINUE
      ENDIF
      CALL POLE2G (LX,MX,NX,SINTH,COSTH,T)
      CALL POLE2G (LX,MX,NX,SINTH,COSTH,U)
      CALL POLE2V (LX,MX,NX,SINTH,COSTH,V)
      CALL POLE2G (LX,MX, 1,SINTH,COSTH,P)
      RETURN
      END

      SUBROUTINE POLE2G (LX,MX,NX,SINTH,COSTH,G)
      REAL G(LX,MX,NX),GG(144),SINTH(144,144/2),COSTH(144,144/2)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
      K1 = INT(REAL(LX/2)*COS(0.5E0*PI*(REAL(M)-0.5E0)/REAL(MX)))
      DO 12 N=1,NX
        SUM = 0.0E0
      DO 21 L=1,LX
        SUM = SUM + G(L,M,N)
   21 CONTINUE
      DO 22 L=1,LX
        GG(L) = SUM/REAL(LX)
   22 CONTINUE
      DO 30 K=1,K1
        SUMS1 = 0.0E0
        SUMS0 = 0.0E0
        SUMC1 = 0.0E0
        SUMC0 = 0.0E0
      DO 31 L=1,LX
        SUMS1 = SUMS1 + G(L,M,N) * SINTH(L,K)
        SUMS0 = SUMS0 + SINTH(L,K)**2
        SUMC1 = SUMC1 + G(L,M,N) * COSTH(L,K)
        SUMC0 = SUMC0 + COSTH(L,K)**2
   31 CONTINUE
        GS = SUMS1/SUMS0
        GC = SUMC1/SUMC0
      DO 32 L=1,LX
        GG(L) = GG(L) + GS * SINTH(L,K)
        GG(L) = GG(L) + GC * COSTH(L,K)
   32 CONTINUE
   30 CONTINUE
      DO 41 L=1,LX
        G(L,M,N) = GG(L)
   41 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE POLE2V (LX,MX,NX,SINTH,COSTH,G)
      REAL G(LX,MX,NX),GG(144),SINTH(144,144/2),COSTH(144,144/2)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
      K1 = INT(REAL(LX/2)*COS(0.5E0*PI*REAL(M-1)/REAL(MX)))
      DO 12 N=1,NX
        SUM = 0.0E0
      DO 21 L=1,LX
        SUM = SUM + G(L,M,N)
   21 CONTINUE
      DO 22 L=1,LX
        GG(L) = SUM/REAL(LX)
   22 CONTINUE
      DO 30 K=1,K1
        SUMS1 = 0.0E0
        SUMS0 = 0.0E0
        SUMC1 = 0.0E0
        SUMC0 = 0.0E0
      DO 31 L=1,LX
        SUMS1 = SUMS1 + G(L,M,N) * SINTH(L,K)
        SUMS0 = SUMS0 + SINTH(L,K)**2
        SUMC1 = SUMC1 + G(L,M,N) * COSTH(L,K)
        SUMC0 = SUMC0 + COSTH(L,K)**2
   31 CONTINUE
        GS = SUMS1/SUMS0
        GC = SUMC1/SUMC0
      DO 32 L=1,LX
        GG(L) = GG(L) + GS * SINTH(L,K)
        GG(L) = GG(L) + GC * COSTH(L,K)
   32 CONTINUE
   30 CONTINUE
      DO 41 L=1,LX
        G(L,M,N) = GG(L)
   41 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================
C   OUTPUT
C ========================================

      SUBROUTINE OFLOPN (LX,MX,NX)
      CHARACTER OUTFL*80
      WRITE(6,*) 'Output files are automatically designated:'
  101 FORMAT (1X,A21,1X,A9)
      OUTFL = 'U1.dat'
      WRITE(6,101) 'Zonal wind          =',OUTFL
      OPEN(11,FILE=OUTFL,STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*(MX+1)*NX)
      OUTFL = 'V1.dat'
      WRITE(6,101) 'Meridional wind     =',OUTFL
      OPEN(12,FILE=OUTFL,STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*(MX+1)*NX)
      OUTFL = 'T1.dat'
      WRITE(6,101) 'Potential temp.     =',OUTFL
      OPEN(13,FILE=OUTFL,STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*(MX+1)*NX)
      OUTFL = 'Z1.dat'
      WRITE(6,101) 'Geopotential height =',OUTFL
      OPEN(14,FILE=OUTFL,STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*(MX+1)*NX)
      OUTFL = 'P1.dat'
      WRITE(6,101) 'Surface pressure    =',OUTFL
      OPEN(15,FILE=OUTFL,STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*(MX+1))
      WRITE(6,*) '===================================='
      RETURN
      END

      SUBROUTINE WRDAT (LX,MX,NX,I,U,V,T,Z,P,GG,PP)
      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),Z(LX,MX,NX),P(LX,MX),
     +     GG(LX,MX+1,NX),PP(LX,MX+1)
      CALL WRCNVU (LX,MX,NX,U,GG)
      WRITE(11,REC=I) GG
      CALL WRCNVV (LX,MX,NX,V,GG)
      WRITE(12,REC=I) GG
      CALL WRCNVG (LX,MX,NX,T,GG)
      WRITE(13,REC=I) GG
      CALL WRCNVG (LX,MX,NX,Z,GG)
      WRITE(14,REC=I) GG
      CALL WRCNVG (LX,MX, 1,P,PP)
      WRITE(15,REC=I) PP
      RETURN
      END

      SUBROUTINE WRCNVG (LX,MX,NX,G,GG)
      REAL G(LX,MX,NX),GG(LX,MX+1,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX+1
      M1 = MAX0(M-1, 1)
      M2 = MIN0(M  ,MX)
      DO 13 L=1,LX
      L1 = MOD(LX+L-2,LX)+1
      L2 = MOD(LX+L-1,LX)+1
        GG(L,M,N) = 0.25E0*(G(L1,M1,N)+G(L2,M1,N)+G(L1,M2,N)+G(L2,M2,N))
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE WRCNVU (LX,MX,NX,G,GG)
      REAL G(LX,MX,NX),GG(LX,MX+1,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX+1
      M1 = MAX0(M-1, 1)
      M2 = MIN0(M  ,MX)
      DO 13 L=1,LX
      LL = MOD(LX+L-1,LX)+1
        GG(L,M,N) = 0.5E0*(G(LL,M1,N)+G(LL,M2,N))
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE WRCNVV (LX,MX,NX,G,GG)
      REAL G(LX,MX,NX),GG(LX,MX+1,NX)
      DO 11 N=1,NX
      DO 20 L=1,LX
        GG(L,   1,N) = 0.0E0
        GG(L,MX+1,N) = 0.0E0
   20 CONTINUE
      DO 21 M=2,MX
      M1 = M-1
      M2 = M
      DO 22 L=1,LX
      L1 = MOD(LX+L-2,LX)+1
      L2 = MOD(LX+L-1,LX)+1
        GG(L,M,N) = 0.25E0*(G(L1,M1,N)+G(L2,M1,N)+G(L1,M2,N)+G(L2,M2,N))
   22 CONTINUE
   21 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================
C   INITIAL CONDITIONS/FORCINGS
C ========================================

      SUBROUTINE FORCE1 (LX,MX,NX,DZ,B0,A0,R0,Z1,Z2,C,DG,DGP)
      REAL DZ(NX),DG(LX,MX,NX),DGP(LX,MX,NX)
      WRITE(6,*) 'Initial perturbation:'
      WRITE(6,'(1X,A3,1X,F7.2,1X,A5)') 'Y =',B0,'deg.N'
      WRITE(6,'(1X,A3,1X,F7.2,1X,A5)') 'X =',A0,'deg.E'
      WRITE(6,'(1X,A3,1X,F7.2,1X,A4)') 'R =',R0,'deg.'
      WRITE(6,'(1X,A6,1X,F8.3,1X,A1)') 'Max. =',C,'K'
      WRITE(6,*) '===================================='
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      CALL VECT  (PI/REAL(180)*A0,PI/REAL(180)*B0,X0,Y0,Z0)
      DO 11 M=1,MX
      B = REAL(90)*(REAL(M)-0.5E0)/REAL(MX)
      DO 12 L=1,LX
      A = REAL(360)*(REAL(L)-0.5E0)/REAL(LX)
      CALL VECT (PI/REAL(180)*A,PI/REAL(180)*B,X,Y,Z)
      P = X0*X + Y0*Y + Z0*Z
      R = REAL(180)/PI * ACOS(P)
      IF (R.LT.R0) THEN
        GH = COS(0.5E0*PI*R/R0)**2
      DO 21 N=1,NX
      CALL SIGMA (NX,DZ,N,Z)
      IF ((Z.GE.Z1).AND.(Z.LE.Z2)) THEN
        GV = 0.5E0*PI*( SIN(PI*(Z-Z1)/(Z2-Z1))
     +                 +0.5E0*SIN(0.2E1*PI*(Z-Z1)/(Z2-Z1)))
        DG (L,M,N) = DG (L,M,N) + C * GH * GV
        DGP(L,M,N) = DGP(L,M,N) + C * GH * GV
      ENDIF
   21 CONTINUE
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE FORCE2 (LX,MX,NX,DZ,B0,A0,R0,Z1,Z2,C,DG)
      REAL DZ(NX),DG(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      CALL VECT  (PI/REAL(180)*A0,PI/REAL(180)*B0,X0,Y0,Z0)
      DO 11 M=1,MX
      B = REAL(90)*(REAL(M)-0.5E0)/REAL(MX)
      DO 12 L=1,LX
      A = REAL(360)*(REAL(L)-0.5E0)/REAL(LX)
      CALL VECT (PI/REAL(180)*A,PI/REAL(180)*B,X,Y,Z)
      P = X0*X + Y0*Y + Z0*Z
      R = REAL(180)/PI * ACOS(P)
      IF (R.LT.R0) THEN
        GH = COS(0.5E0*PI*R/R0)**2
      DO 21 N=1,NX
      CALL SIGMA (NX,DZ,N,Z)
      IF ((Z.GE.Z1).AND.(Z.LE.Z2)) THEN
        GV = 0.5E0*PI*( SIN(PI*(Z-Z1)/(Z2-Z1))
     +                 +0.5E0*SIN(0.2E1*PI*(Z-Z1)/(Z2-Z1)))
        DG (L,M,N) = DG (L,M,N) + C * GH * GV
      ENDIF
   21 CONTINUE
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE VECT (A,B,X,Y,Z)
      X = COS(B) * COS(A)
      Y = COS(B) * SIN(A)
      Z = SIN(B)
      RETURN
      END
