C ========================================
C   INITIAL CONDITIONS
C ========================================

      SUBROUTINE INITZ (LX,MX,NX,U,V,T,P,UP,VP,TP,PP)
      REAL U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX)
C  P : Surface pressure of the anomaly field.
C  U : Zonal wind of the anomaly field.
C  V : Meridional wind of the anomaly field.
C  T : Temperature of the anomaly field.
C  PP: Surface pressure at the previous time step, of the anomaly field.
C  UP: Zonal wind at the previous time step, of the anomaly field.
C  VP: Meridional wind at the previous time step, of the anomaly field.
C  TP: Temperature at the previous time step, of the anomaly field.
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        U (L,M,N) = 0.0E0
        V (L,M,N) = 0.0E0
        T (L,M,N) = 0.0E0
        P (L,M) = 0.0E0
        UP(L,M,N) = U(L,M,N)
        VP(L,M,N) = V(L,M,N)
        TP(L,M,N) = T(L,M,N)
        PP(L,M) = P(L,M)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE CONST (PI,RE,OM,GC,RC,T00,P00)
      PI = 0.31415926535E1
      RE = 0.637E7
      OM = 0.727E-4
      GC = 0.9807E1
      RC = 0.2870E3
      T00 = 0.298E3
      P00 = 0.101325E6
      RETURN
      END

      SUBROUTINE SIGMA (NX,DZ,N,Z)
      REAL DZ(NX)
      IF (N.GE.NX+1) THEN
        Z = 1.0E0
        RETURN
      ENDIF
      IF (N.LE.0) THEN
        Z = 1.0E0
      DO 10 NN=NX,1,-1
        Z = Z - DZ(NN)
   10 CONTINUE
        RETURN
      ENDIF
        Z = 1.0E0
      DO 11 NN=NX,N+1,-1
        Z = Z - DZ(NN)
   11 CONTINUE
        Z = Z - 0.5E0*DZ(N)
      RETURN
      END

C ========================================
C   DIAGNOSES OF DEPENDENT VARIABLES
C ========================================

      SUBROUTINE DIAGW (LX,MX,NX,DX,DY,DZ,CM,PTOP,P0,U0,V0,P,U,V,W,PA)
      REAL DX(MX),DY(MX),DZ(NX),
     +     P0(LX,MX),U0(LX,MX,NX),V0(LX,MX,NX),
     +     P (LX,MX),U (LX,MX,NX),V (LX,MX,NX),
     +     W(LX,MX,NX),PA(LX,MX)
C  P0: Surface pressure of the basic field.
C  U0: Zonal wind of the basic field.
C  V0: Meridional wind of the basic field.
C  P : Surface pressure of the anomaly field.
C  U : Zonal wind of the anomaly field.
C  V : Meridional wind of the anomaly field.
C  W : Diagnosed vertical sigma-velocity of the anomaly field.
C  PA: Sum of P0 and P. Dummy.
      CHARACTER CM*1
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        W(L,M,N) = 0.0E0
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      IF (CM.EQ.'I') THEN
      CALL DIAGW1 (LX,MX,NX,DX,DY,DZ,PTOP ,P0,P0,U0,V0,W)
      ELSE
      IF (CM.NE.'N') THEN
      CALL DIAGW1 (LX,MX,NX,DX,DY,DZ,PTOP ,P0,P0,U ,V ,W)
      CALL DIAGW1 (LX,MX,NX,DX,DY,DZ,0.0E0,P0,P ,U0,V0,W)
      IF (CM.EQ.'M') THEN
      CALL DIAGW1 (LX,MX,NX,DX,DY,DZ,0.0E0,P0,P ,U ,V ,W)
      ENDIF
      ELSE
      DO 21 M=1,MX
      DO 22 L=1,LX
        PA(L,M) = P0(L,M) + P(L,M)
   22 CONTINUE
   21 CONTINUE
      CALL DIAGW1 (LX,MX,NX,DX,DY,DZ,PTOP,PA,PA,U0,V0,W)
      CALL DIAGW1 (LX,MX,NX,DX,DY,DZ,PTOP,PA,PA,U ,V ,W)
      ENDIF
      ENDIF
      RETURN
      END

      SUBROUTINE DIAGW1 (LX,MX,NX,DX,DY,DZ,PTOP,PO,P,U,V,W)
      REAL DX(MX),DY(MX),DZ(NX),PO(LX,MX),P(LX,MX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX)
      CALL SIGMA (NX,DZ,NX+1,Z1)
      CALL SIGMA (NX,DZ,   0,Z0)
      DO 11 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      M1 = MAX0(M-1, 1)
      M2 = MIN0(M+1,MX)
      DO 12 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
        W0 = 0.0E0
      DO 21 N=1,NX
      DX1 = 0.5E0*(DX(M)+DX(M1))
      DX2 = 0.5E0*(DX(M)+DX(M2))
      DXDY = DX(M) * DY(M)
      DXDYI = 1.0E0/DXDY
      FX1 = DY(M) * U(L ,M ,N) * (0.5E0*(P(L ,M )+P(LA,M ))-PTOP)
      FX2 = DY(M) * U(LB,M ,N) * (0.5E0*(P(LB,M )+P(L ,M ))-PTOP)
      FY1 = DX1   * V(L ,M ,N) * (0.5E0*(P(L ,M )+P(L ,MA))-PTOP)
      FY2 = DX2   * V(L ,MB,N) * (0.5E0*(P(L ,MB)+P(L ,M ))-PTOP)
        W0 = W0 + (FX2-FX1+FY2-FY1)*DXDYI * DZ(N) / (Z1-Z0)
   21 CONTINUE
      DO 31 N=1,NX-1
      CALL SIGMA (NX,DZ,N,Z)
      Z = Z + 0.5E0*DZ(N)
        W1 = 0.0E0
      DO 32 NN=1,N
      FX1 = DY(M) * U(L ,M ,NN) * (0.5E0*(P(L ,M )+P(LA,M ))-PTOP)
      FX2 = DY(M) * U(LB,M ,NN) * (0.5E0*(P(LB,M )+P(L ,M ))-PTOP)
      FY1 = DX1   * V(L ,M ,NN) * (0.5E0*(P(L ,M )+P(L ,MA))-PTOP)
      FY2 = DX2   * V(L ,MB,NN) * (0.5E0*(P(L ,MB)+P(L ,M ))-PTOP)
        W1 = W1 - (FX2-FX1+FY2-FY1)*DXDYI * DZ(NN) / (Z1-Z0)
   32 CONTINUE
        W(L,M,N+1)= W(L,M,N+1) + (W1+(Z-Z0)/(Z1-Z0)*W0) / PO(L,M)
   31 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE DIAGX (LX,MX,NX,DZ,PTOP,P,EX)
      REAL DZ(NX),P(LX,MX),EX(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
C  P : Surface pressure.
C  EX: Exner function (ratio of temperature to potential
C      temperature). 
      CALL SIGMA (NX,DZ,NX+1,Z1)
      CALL SIGMA (NX,DZ,   0,Z0)
      DO 11 M=1,MX
      DO 12 L=1,LX
      DO 21 N=NX,1,-1
      CALL SIGMA (NX,DZ,N,Z)
        PR = (P(L,M)*(Z-Z0)+PTOP*(Z1-Z))/(Z1-Z0)
        EX(L,M,N) = (PR/1.0E5)**(0.2E1/0.7E1)
   21 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE DIAGH (LX,MX,NX,DZ,CM,TPG,PTOP,P,EX,T,H)
      REAL DZ(NX),TPG(LX,MX),P(LX,MX),EX(LX,MX,NX),T(LX,MX,NX),
     +     H(LX,MX,NX)
      CHARACTER CM*1
C  EX: Exner function (ratio of temperature to potential
C      temperature). 
C  T : Temperature.
C  H : Geopotential height of the sigma-surface.
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      CALL SIGMA (NX,DZ,NX+1,Z1)
      CALL SIGMA (NX,DZ,   0,Z0)
      DO 11 M=1,MX
      DO 12 L=1,LX
        HH = 0.0E0
      IF (CM.EQ.'I') HH = TPG(L,M)
      DO 21 N=NX,1,-1
      CALL SIGMA (NX,DZ,N,Z)
        PR = (P(L,M)*(Z-Z0)+PTOP*(Z1-Z))/(Z1-Z0)
        DPR = (P(L,M)-PTOP)*DZ(N)/(Z1-Z0)
        TM = T(L,M,N) * EX(L,M,N)
        DH = RC * TM / GC * DPR / PR
        H(L,M,N) = HH + 0.5*DH
        HH = HH + DH
   21 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE DIAGR (LX,MX,NX,DZ,PTOP,P,T,RTG)
      REAL DZ(NX),P(LX,MX),T(LX,MX,NX),RTG(LX,MX,NX)
C  P  : Surface pressure.
C  T  : Temperature.
C  RTG: Scale height.
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      CALL SIGMA (NX,DZ,NX+1,Z1)
      CALL SIGMA (NX,DZ,   0,Z0)
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
      CALL SIGMA (NX,DZ,N,Z)
        PR = (P(L,M)*(Z-Z0)+PTOP*(Z1-Z))/(Z1-Z0)
        TM = T(L,M,N)*(PR/1.0E5)**(0.2E1/0.7E1)
        RTG(L,M,N) = RC*TM/GC
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE DIAGZ (LX,MX,NX,RTG,P0,P,H,Z)
      REAL RTG(LX,MX,NX),P0(LX,MX),P(LX,MX),H(LX,MX,NX),Z(LX,MX,NX)
C  P0: Surface pressure of the basic field.
C  P : Surface pressure of the anomaly field.
C  H : Geopotential height of the sigma-surface.
C  Z : Diagnosed geopotential height of the pressure surface.
      DO 11 M=1,MX
      DO 12 L=1,LX
      DO 21 N=1,NX
        Z(L,M,N) = H(L,M,N) + RTG(L,M,N) * P(L,M) / P0(L,M)
   21 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================
C   DYNAMICAL PROCESSES
C ========================================

      SUBROUTINE CNVRG (LX,MX,NX,DX,DY,DZ,CM,PTOP,P0,U0,V0,P,U,V,DP) 
      REAL DX(MX),DY(MX),DZ(NX),
     +     P0(LX,MX),U0(LX,MX,NX),V0(LX,MX,NX),
     +     P (LX,MX),U (LX,MX,NX),V (LX,MX,NX),DP(LX,MX)
      CHARACTER CM*1
C  P0: Surface pressure of the basic field.
C  U0: Zonal wind of the basic field.
C  V0: Meridional wind of the basic field.
C  P : Surface pressure of the anomaly field.
C  U : Zonal wind of the anomaly field.
C  V : Meridional wind of the anomaly field.
C  DP: Time-derivative of surface pressure.
      CALL CNVRG1 (LX,MX,NX,DX,DY,DZ,PTOP ,U ,V ,P0,DP)
      CALL CNVRG1 (LX,MX,NX,DX,DY,DZ,0.0E0,U0,V0,P ,DP)
      IF (CM.EQ.'M') THEN
      CALL CNVRG1 (LX,MX,NX,DX,DY,DZ,0.0E0,U ,V ,P ,DP)
      ENDIF
      IF (CM.EQ.'N') THEN
      CALL CNVRG1 (LX,MX,NX,DX,DY,DZ,PTOP ,U0,V0,P0,DP)
      CALL CNVRG1 (LX,MX,NX,DX,DY,DZ,0.0E0,U ,V ,P ,DP)
      ENDIF
      RETURN
      END

      SUBROUTINE CNVRG1 (LX,MX,NX,DX,DY,DZ,PTOP,U,V,P,DP)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),P(LX,MX),DP(LX,MX)
      CALL SIGMA (NX,DZ,NX+1,Z1)
      CALL SIGMA (NX,DZ,   0,Z0)
      DO 11 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      DXDY = DX(M) * DY(M)
      DXDYI = 1.0E0/DXDY
      DO 12 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      DO 13 N=1,NX
      FX1 = DY(M) * U(L ,M ,N) * (0.5E0*(P(L ,M )+P(LA,M ))-PTOP)
      FX2 = DY(M) * U(LB,M ,N) * (0.5E0*(P(LB,M )+P(L ,M ))-PTOP)
      FY1 = DX1   * V(L ,M ,N) * (0.5E0*(P(L ,M )+P(L ,MA))-PTOP)
      FY2 = DX2   * V(L ,MB,N) * (0.5E0*(P(L ,MB)+P(L ,M ))-PTOP)
        DP(L,M) = DP(L,M)
     +    - (FX2-FX1+FY2-FY1)*DXDYI * DZ(N) / (Z1-Z0)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE ADVEC (LX,MX,NX,DX,DY,DZ,CM,
     +                  U0,V0,W0,T0,U,V,W,T,DU,DV,DT)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U0(LX,MX,NX),V0(LX,MX,NX),W0(LX,MX,NX),T0(LX,MX,NX),
     +     U (LX,MX,NX),V (LX,MX,NX),W (LX,MX,NX),T (LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      CHARACTER CM*1
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U0,V0,W0,U ,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U0,V0,W0,V ,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U0,V0,W0,T ,DT)
C      CALL ADVECH (LX,MX,NX,DX,DY,DZ,U0,V0,W0,T ,DT)
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U ,V ,W ,U0,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U ,V ,W ,V0,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U ,V ,W ,T0,DT)
      IF (CM.EQ.'M') THEN
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U ,V ,W ,U ,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U ,V ,W ,V ,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U ,V ,W ,T ,DT)
      ENDIF
      IF (CM.EQ.'N') THEN
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U0,V0,W0,U0,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U0,V0,W0,V0,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U0,V0,W0,T0,DT)
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U ,V ,W ,U ,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U ,V ,W ,V ,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U ,V ,W ,T ,DT)
      ENDIF
      RETURN
      END

      SUBROUTINE ADVECG (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      SX1 = DY(M) * DZ(N)
      SX2 = SX1
      SY1 = DX1 * DZ(N)
      SY2 = DX2 * DZ(N)
      SZ1 = DX(M) * DY(M)
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DXDYDZI = 1.0E0/DXDYDZ
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      FX1 = SX1 * U(L ,M ,N ) * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U(LB,M ,N ) * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V(L ,M ,N ) * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V(L ,MB,N ) * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W(L ,M ,N ) * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W(L ,M ,NB) * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) * DXDYDZI
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ADVECH (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX-1
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      SX1 = DY(M) * DZ(N)
      SX2 = SX1
      SY1 = DX1 * DZ(N)
      SY2 = DX2 * DZ(N)
      SZ1 = DX(M) * DY(M)
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DXDYDZI = 1.0E0/DXDYDZ
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      FX1 = SX1 * U(L ,M ,N ) * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U(LB,M ,N ) * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V(L ,M ,N ) * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V(L ,MB,N ) * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W(L ,M ,N ) * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W(L ,M ,NB) * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) * DXDYDZI
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ADVECB (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX-1
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      SX1 = DY(M) * DZ(N)
      SX2 = SX1
      SY1 = DX1 * DZ(N)
      SY2 = DX2 * DZ(N)
      SZ1 = DX(M) * DY(M)
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DXDYDZI = 1.0E0/DXDYDZ
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      FX1 = SX1 * U(L ,M ,N ) * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U(LB,M ,N ) * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V(L ,M ,N ) * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V(L ,MB,N ) * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W(L ,M ,N ) * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W(L ,M ,NB) * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) * DXDYDZI
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ADVECU (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      SX1 = DY(M) * DZ(N)
      SX2 = SX1
      SY1 = DX1 * DZ(N)
      SY2 = DX2 * DZ(N)
      SZ1 = DX(M) * DY(M)
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DXDYDZI = 1.0E0/DXDYDZ
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      U1 = 0.5E0*(U(LA,M ,N )+U(L ,M ,N ))
      U2 = 0.5E0*(U(L ,M ,N )+U(LB,M ,N ))
      V1 = 0.5E0*(V(LA,M ,N )+V(L ,M ,N ))
      V2 = 0.5E0*(V(LA,MB,N )+V(L ,MB,N ))
      W1 = 0.5E0*(W(LA,M ,N )+W(L ,M ,N ))
      W2 = 0.5E0*(W(LA,M ,NB)+W(L ,M ,NB))
      FX1 = SX1 * U1 * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U2 * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V1 * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V2 * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W1 * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W2 * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) * DXDYDZI
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ADVECV (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=2,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX0 = 0.5E0*(DX(M)+DX(MA))
      DY0 = 0.5E0*(DY(M)+DY(MA))
      SX1 = DY0 * DZ(N)
      SX2 = SX1
      SY1 = DX(MA) * DZ(N)
      SY2 = DX(M)  * DZ(N)
      SZ1 = DX0 * DY0
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DXDYDZI = 1.0E0/DXDYDZ
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      U1 = 0.5E0*(U(L ,MA,N )+U(L ,M ,N ))
      U2 = 0.5E0*(U(LB,MA,N )+U(LB,M ,N ))
      V1 = 0.5E0*(V(L ,MA,N )+V(L ,M ,N ))
      V2 = 0.5E0*(V(L ,M ,N )+V(L ,MB,N ))
      W1 = 0.5E0*(W(L ,MA,N )+W(L ,M ,N ))
      W2 = 0.5E0*(W(L ,MA,NB)+W(L ,M ,NB))
      FX1 = SX1 * U1 * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U2 * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V1 * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V2 * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W1 * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W2 * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) * DXDYDZI
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE PGRAD (LX,MX,NX,DX,DY,DZ,
     +                  CM,PTOP,P0,H0,P,H,RTG,DU,DV)
      REAL DX(MX),DY(MX),DZ(NX),
     +     P0(LX,MX),P(LX,MX),H0(LX,MX,NX),H(LX,MX,NX),
     +     RTG(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX)
      CHARACTER CM*1
      CALL PGRAD1 (LX,MX,NX,DX,DY,DZ,PTOP,P0,P,H,RTG,DU,DV)
      IF (CM.EQ.'N') THEN
      CALL PGRAD1 (LX,MX,NX,DX,DY,DZ,PTOP,P0,P0,H0,RTG,DU,DV)
      ENDIF
      RETURN
      END

      SUBROUTINE PGRAD1 (LX,MX,NX,DX,DY,DZ,PTOP,P0,P,H,RTG,DU,DV)
      REAL DX(MX),DY(MX),DZ(NX),
     +     P0(LX,MX),P(LX,MX),H(LX,MX,NX),RTG(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
C      CALL SIGMA (NX,DZ,NX+1,Z1)
C      CALL SIGMA (NX,DZ,   0,Z0)
C      Z1Z0I = 1.0E0/(Z1-Z0)
      DO 11 N=1,NX
C      CALL SIGMA (NX,DZ,N,Z)
C      ZR = (Z-Z0)*Z1Z0I
C      C1 = (Z1-Z)*Z1Z0I
C      C2 = (Z-Z0)*Z1Z0I
      DO 12 M=1,MX
      DXI = 1.0E0/DX(M)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      PP = 0.5E0*(P0(LA,M)+P0(L,M))
        DU(L,M,N) = DU(L,M,N)
     +    - GC * (  (H(L,M,N)-H(LA,M,N))
     +            + RTG(L,M,N)*(P(L,M)-P(LA,M))/PP)
     +         * DXI
C      PP = 0.5E0*(P0(L,MA)+P0(L,M))
C      PP = C1*PTOP+C2*PP
C        DU(L,M,N) = DU(L,M,N)
C     +    - GC * (  (H(L,M,N)-H(LA,M,N))
C     +            + RTG(L,M,N)*ZR*(P(L,M)-P(LA,M))/PP)
C     +         * DXI
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
C      CALL SIGMA (NX,DZ,N,Z)
C      ZR = (Z-Z0)*Z1Z0I
C      C1 = (Z1-Z)*Z1Z0I
C      C2 = (Z-Z0)*Z1Z0I
      DO 22 M=2,MX
      MA = MOD(M-1+MX-1,MX)+1
      DY0 = 0.5E0*(DY(MA)+DY(M))
      DYI = 1.0E0/DY0
      DO 23 L=1,LX
      PP = 0.5E0*(P0(L,MA)+P0(L,M))
        DV(L,M,N) = DV(L,M,N)
     +    - GC * (  (H(L,M,N)-H(L,MA,N))
     +            + RTG(L,M,N)*(P(L,M)-P(L,MA))/PP)
     +         * DYI
C      PP = 0.5E0*(P0(L,MA)+P0(L,M))
C      PP = C1*PTOP+C2*PP
C        DV(L,M,N) = DV(L,M,N)
C     +    - GC * (  (H(L,M,N)-H(L,MA,N))
C     +            + RTG(L,M,N)*ZR*(P(L,M)-P(L,MA))/PP)
C     +         * DYI
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE CORIO (LX,MX,NX,CG,CM,FC,U0,V0,U,V,DU,DV)
      REAL FC(MX),U0(LX,MX,NX),V0(LX,MX,NX),U(LX,MX,NX),V(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX)
      CHARACTER CG*1,CM*1
      IF (CM.NE.'N') THEN
        CALL CORIOL (LX,MX,NX,CG,FC,U0,V0,U,V,DU,DV)
      ELSE
        CALL CORION (LX,MX,NX,CG,FC,U0,V0,U,V,DU,DV)
      ENDIF
      RETURN
      END
 
      SUBROUTINE CORIOL (LX,MX,NX,CG,FC,U0,V0,U,V,DU,DV)
      REAL FC(MX),U0(LX,MX,NX),V0(LX,MX,NX),U(LX,MX,NX),V(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX)
      CHARACTER CG*1
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 N=1,NX
      DO 12 M=1,MX
      IF (CG.EQ.'H') TH = 0.5E0*PI*(REAL(M)-0.5E0)/REAL(MX)
      IF (CG.EQ.'G') TH = PI*((REAL(M)-0.5E0)/REAL(MX)-0.5E0)
      TANTH = TAN(TH)
      MB = MOD(M+1+MX-1,MX)+1
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      FF = FC(M) + U0(L,M,N) / RE * TANTH
      VV = 0.25E0*(V(LA,M,N)+V(L,M,N)+V(LA,MB,N)+V(L,MB,N))
        DU(L,M,N) = DU(L,M,N) + FF * VV
      FF = U(L,M,N) / RE * TANTH
      VV = 0.25E0*(V0(LA,M,N)+V0(L,M,N)+V0(LA,MB,N)+V0(L,MB,N))
        DU(L,M,N) = DU(L,M,N) + FF * VV
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      DO 22 M=2,MX
      IF (CG.EQ.'H') TH = 0.5E0*PI*REAL(M-1)/REAL(MX)
      IF (CG.EQ.'G') TH = PI*(REAL(M-1)/REAL(MX)-0.5E0)
      TANTH = TAN(TH)
      MA = MOD(M-1+MX-1,MX)+1
      DO 23 L=1,LX
      LB = MOD(L+1+LX-1,LX)+1
      FF = 0.5E0*(FC(MA)+FC(M))
     +     + 0.25E0*(U0(L,MA,N)+U0(LB,MA,N)+U0(L,M,N)+U0(LB,M,N))
     +       / RE * TANTH
      UU = 0.25E0*(U(L,MA,N)+U(LB,MA,N)+U(L,M,N)+U(LB,M,N))
        DV(L,M,N) = DV(L,M,N) - FF * UU
      FF = 0.25E0*(U(L,MA,N)+U(LB,MA,N)+U(L,M,N)+U(LB,M,N))
     +       / RE * TANTH
      UU = 0.25E0*(U0(L,MA,N)+U0(LB,MA,N)+U0(L,M,N)+U0(LB,M,N))
        DV(L,M,N) = DV(L,M,N) - FF * UU
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

      SUBROUTINE CORION (LX,MX,NX,CG,FC,U0,V0,U,V,DU,DV)
      REAL FC(MX),U0(LX,MX,NX),V0(LX,MX,NX),U(LX,MX,NX),V(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX)
      CHARACTER CG*1
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 N=1,NX
      DO 12 M=1,MX
      IF (CG.EQ.'H') TH = 0.5E0*PI*(REAL(M)-0.5E0)/REAL(MX)
      IF (CG.EQ.'G') TH = PI*((REAL(M)-0.5E0)/REAL(MX)-0.5E0)
      TANTH = TAN(TH)
      MB = MOD(M+1+MX-1,MX)+1
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      FF = FC(M) + (U0(L,M,N)+U(L,M,N)) / RE * TANTH
      VV = 0.25E0*( V0(LA,M,N)+V0(L,M,N)+V0(LA,MB,N)+V0(L,MB,N)
     +             +V (LA,M,N)+V (L,M,N)+V (LA,MB,N)+V (L,MB,N))
        DU(L,M,N) = DU(L,M,N) + FF * VV
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      DO 22 M=2,MX
      IF (CG.EQ.'H') TH = 0.5E0*PI*REAL(M-1)/REAL(MX)
      IF (CG.EQ.'G') TH = PI*(REAL(M-1)/REAL(MX)-0.5E0)
      TANTH = TAN(TH)
      MA = MOD(M-1+MX-1,MX)+1
      DO 23 L=1,LX
      LB = MOD(L+1+LX-1,LX)+1
      FF = 0.5E0*(FC(MA)+FC(M))
     +     + 0.25E0*( U0(L,MA,N)+U0(LB,MA,N)+U0(L,M,N)+U0(LB,M,N)
     +               +U (L,MA,N)+U (LB,MA,N)+U (L,M,N)+U (LB,M,N))
     +       / RE * TANTH
      UU = 0.25E0*( U0(L,MA,N)+U0(LB,MA,N)+U0(L,M,N)+U0(LB,M,N)
     +             +U (L,MA,N)+U (LB,MA,N)+U (L,M,N)+U (LB,M,N))
        DV(L,M,N) = DV(L,M,N) - FF * UU
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

C ========================================
C   PHYSICAL PROCESSES
C ========================================

      SUBROUTINE DIFFU (LX,MX,NX,DX,DY,DZ,RTG,CM,
     +                  U0,V0,T0,U,V,T,DU,DV,DT)
      REAL DX(MX),DY(MX),DZ(NX),RTG(LX,MX,NX),
     +     U0(LX,MX,NX),V0(LX,MX,NX),T0(LX,MX,NX),
     +     U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      CHARACTER CM*1
      DFH = 1.0E5
      DFV = 1.0E0
      CALL DIFFU1 (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,U ,V ,T ,DU,DV,DT)
      IF (CM.EQ.'N') THEN
      CALL DIFFU1 (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,U0,V0,T0,DU,DV,DT)
      ENDIF
      RETURN
      END

      SUBROUTINE DIFFU1 (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,U,V,T,DU,DV,DT)
      REAL DX(MX),DY(MX),DZ(NX),RTG(LX,MX,NX),
     +     U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      CALL DIFFUU (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,U,DU)
      CALL DIFFUV (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,V,DV)
      CALL DIFFUG (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,T,DT)
      RETURN
      END

      SUBROUTINE DIFFUG (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),RTG(LX,MX,NX),G(LX,MX,NX),DG(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 N=1,NX
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      M1 = MAX(M-1, 1)
      M2 = MIN(M+1,MX)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
        DG(L,M,N) = DG(L,M,N)
     +    + DFH * (G(LA,M,N)+G(LB,M,N)-0.2E1*G(L,M,N))
     +      / DX(M)**2
     +    + DFH * (G(L,M1,N)+G(L,M2,N)-0.2E1*G(L,M,N))
     +      / DY(M)**2
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      N1 = MAX(N-1, 1)
      N2 = MIN(N+1,NX)
      CALL SIGMA (NX,DZ,N ,Z )
      CALL SIGMA (NX,DZ,N1,Z1)
      CALL SIGMA (NX,DZ,N2,Z2)
      DZ1 = 0.5E0*(DZ(N1)+DZ(N ))
      DZ2 = 0.5E0*(DZ(N )+DZ(N2))
      DO 22 M=1,MX
      DO 23 L=1,LX
      DFV0 = DFV / RTG(L,M,N)
      DFV1 = DFV0 * (0.5E0*(Z+Z1))
      DFV2 = DFV0 * (0.5E0*(Z+Z2))
        DG(L,M,N) = DG(L,M,N)
     +    + (  DFV2*(G(L,M,N2)-G(L,M,N))/DZ2
     +       - DFV1*(G(L,M,N1)-G(L,M,N))/DZ1)
     +      / (DZ(N)*RTG(L,M,N)) * Z
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

      SUBROUTINE DIFFUU (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),RTG(LX,MX,NX),G(LX,MX,NX),DG(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 N=1,NX
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      M1 = MAX(M-1, 1)
      M2 = MIN(M+1,MX)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
        DG(L,M,N) = DG(L,M,N)
     +    + DFH * (G(LA,M,N)+G(LB,M,N)-0.2E1*G(L,M,N))
     +      / DX(M)**2
     +    + DFH * (G(L,M1,N)+G(L,M2,N)-0.2E1*G(L,M,N))
     +      / DY(M)**2
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      N1 = MAX(N-1, 1)
      N2 = MIN(N+1,NX)
      CALL SIGMA (NX,DZ,N ,Z )
      CALL SIGMA (NX,DZ,N1,Z1)
      CALL SIGMA (NX,DZ,N2,Z2)
      DZ1 = 0.5E0*(DZ(N1)+DZ(N ))
      DZ2 = 0.5E0*(DZ(N )+DZ(N2))
      DO 22 M=1,MX
      DO 23 L=1,LX
      DFV0 = DFV / RTG(L,M,N)
      DFV1 = DFV0 * (0.5E0*(Z+Z1))
      DFV2 = DFV0 * (0.5E0*(Z+Z2))
      DZI = 1.0E0 / (DZ(N)*RTG(L,M,N))
        DG(L,M,N) = DG(L,M,N)
     +    + (  DFV2*(G(L,M,N2)-G(L,M,N))/DZ2
     +       - DFV1*(G(L,M,N1)-G(L,M,N))/DZ1)
     +      * DZI * Z
      IF (N.EQ.NX) THEN
        DG(L,M,N) = DG(L,M,N)
     +     - DFV2*G(L,M,N)/(0.5E0*DZ2) * DZI * (0.5E0*(Z+Z2))
      ENDIF
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

      SUBROUTINE DIFFUV (LX,MX,NX,DX,DY,DZ,RTG,DFH,DFV,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),RTG(LX,MX,NX),G(LX,MX,NX),DG(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 N=1,NX
      DO 12 M=2,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX0 = 0.5E0*(DX(MA)+DX(M))
      DY0 = 0.5E0*(DY(MA)+DY(M))
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
        DG(L,M,N) = DG(L,M,N)
     +    + DFH * (G(LA,M,N)+G(LB,M,N)-0.2E1*G(L,M,N))
     +      / DX0**2
     +    + DFH * (G(L,MA,N)+G(L,MB,N)-0.2E1*G(L,M,N))
     +      / DY0**2
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      N1 = MAX(N-1, 1)
      N2 = MIN(N+1,NX)
      CALL SIGMA (NX,DZ,N ,Z )
      CALL SIGMA (NX,DZ,N1,Z1)
      CALL SIGMA (NX,DZ,N2,Z2)
      DZ1 = 0.5E0*(DZ(N1)+DZ(N ))
      DZ2 = 0.5E0*(DZ(N )+DZ(N2))
      DO 22 M=2,MX
      DO 23 L=1,LX
      DFV0 = DFV / RTG(L,M,N)
      DFV1 = DFV0 * (0.5E0*(Z+Z1))
      DFV2 = DFV0 * (0.5E0*(Z+Z2))
      DZI = 1.0E0 / (DZ(N)*RTG(L,M,N))
        DG(L,M,N) = DG(L,M,N)
     +    + (  DFV2*(G(L,M,N2)-G(L,M,N))/DZ2
     +       - DFV1*(G(L,M,N1)-G(L,M,N))/DZ1)
     +      * DZI * Z
      IF (N.EQ.NX) THEN
        DG(L,M,N) = DG(L,M,N)
     +     - DFV2*G(L,M,N)/(0.5E0*DZ2) * DZI * (0.5E0*(Z+Z2))
      ENDIF
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE DMPNG (LX,MX,NX,U,V,T,DU,DV,DT)
      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      TAU = 1.0E6
      TAUI = 1.0E0/TAU
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
C        DU(L,M,N) = DU(L,M,N) - TAUI * U(L,M,N)
C        DV(L,M,N) = DV(L,M,N) - TAUI * V(L,M,N)
        DT(L,M,N) = DT(L,M,N) - TAUI * T(L,M,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C      SUBROUTINE DMPNG (LX,MX,NX,U,V,T,DU,DV,DT)
C      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),
C     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
C      TAU  = 0.86400E5*1.0E1
C      TAU2 = 0.86400E5*0.3E1
C      TAU1 = 0.86400E5*0.1E1
C      TAUI  = 1.0E0/TAU
C      TAU2I = 1.0E0/TAU2
C      TAU1I = 1.0E0/TAU1
C      DO 11 N=1,NX
C      DO 12 M=1,MX
C      DO 13 L=1,LX
C      IF (N.EQ.NX) THEN
C        DU(L,M,N) = DU(L,M,N) - TAU1I * U(L,M,N)
C        DV(L,M,N) = DV(L,M,N) - TAU1I * V(L,M,N)
C        DT(L,M,N) = DT(L,M,N) - TAU1I * T(L,M,N)
C      ELSE
C      IF (N.EQ.NX-1) THEN
C        DU(L,M,N) = DU(L,M,N) - TAU2I * U(L,M,N)
C        DV(L,M,N) = DV(L,M,N) - TAU2I * V(L,M,N)
C        DT(L,M,N) = DT(L,M,N) - TAU2I * T(L,M,N)
C      ELSE
C        DU(L,M,N) = DU(L,M,N) - TAUI * U(L,M,N)
C        DV(L,M,N) = DV(L,M,N) - TAUI * V(L,M,N)
C        DT(L,M,N) = DT(L,M,N) - TAUI * T(L,M,N)
C      ENDIF
C      ENDIF
C   13 CONTINUE
C   12 CONTINUE
C   11 CONTINUE
C      RETURN
C      END

C ========================================
C   TEMPORAL EVOLUTION
C ========================================

      SUBROUTINE TENDZ (LX,MX,NX,DU,DV,DT,DP)
      REAL DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX)
      CALL ZERO (LX,MX,NX,DU)
      CALL ZERO (LX,MX,NX,DV)
      CALL ZERO (LX,MX,NX,DT)
      CALL ZERO (LX,MX, 1,DP)
      RETURN
      END

      SUBROUTINE ZERO (LX,MX,NX,G)
      REAL G(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        G(L,M,N) = 0.0E0
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE INTEL (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      REAL U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX)
      CALL INTEL1 (LX,MX,NX,DTM,DU,UP,U)
      CALL INTEL1 (LX,MX,NX,DTM,DV,VP,V)
      CALL INTEL1 (LX,MX,NX,DTM,DT,TP,T)
      CALL INTEL1 (LX,MX, 1,DTM,DP,PP,P)
      RETURN
      END

      SUBROUTINE INTEE (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      REAL U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX)
      CALL INTEE1 (LX,MX,NX,DTM,DU,UP,U)
      CALL INTEE1 (LX,MX,NX,DTM,DV,VP,V)
      CALL INTEE1 (LX,MX,NX,DTM,DT,TP,T)
      CALL INTEE1 (LX,MX, 1,DTM,DP,PP,P)
      RETURN
      END

      SUBROUTINE INTEL1 (LX,MX,NX,DTM,DG,GP,G)
      REAL DG(LX,MX,NX),GP(LX,MX,NX),G(LX,MX,NX)
      EF = 0.5E-1
      EF1 = (1.0E0-0.2E1*EF)/(1.0E0-EF)
      EF2 = EF/(1.0E0-EF)
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        GP1       = GP(L,M,N)
        GP(L,M,N) = EF1*G(L,M,N) + EF2*GP(L,M,N)
        G (L,M,N) = GP1 + 0.2E1*DTM * DG(L,M,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE INTEE1 (LX,MX,NX,DTM,DG,GP,G)
      REAL DG(LX,MX,NX),GP(LX,MX,NX),G(LX,MX,NX)
      EF = 0.5E-1
      EF1 = 1.0E0-EF
      EF2 = EF
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        G (L,M,N) = G(L,M,N) + 0.2E1*DTM * DG(L,M,N)
        GP(L,M,N) = EF1*GP(L,M,N) + EF2*G(L,M,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END
